//go:build !amd64

// !amd64

package tool

import (
	"crypto/aes"
	"crypto/cipher"
	"errors"
	"fmt"
)

var (
	base64Pad          = byte('=')
	base64ReverseTable = [256]int8{
		-2, -2, -2, -2, -2, -2, -2, -2, -2, -1, -1, -2, -2, -1, -2, -2,
		-2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
		-1, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, 62, -2, -2, -2, 63,
		52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -2, -2, -2, -2, -2, -2,
		-2, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
		15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -2, -2, -2, -2, -2,
		-2, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
		41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -2, -2, -2, -2, -2,
		-2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
		-2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
		-2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
		-2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
		-2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
		-2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
		-2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
		-2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
	}
)

// ParseTokenWithAes256 模拟Phalcon解密token
func ParseTokenWithAes256(token, key string) ([]byte, error) {

	var inl = len(token)
	var ik = 0
	var i = 0
	var j = 0
	var padding = 0
	var out = make([]byte, inl)

	for inl > 0 {
		inl--

		ch := token[ik]
		ik++
		if ch == base64Pad {
			padding++
			continue
		}

		ch = uint8(base64ReverseTable[ch])
		if ch < 0 {
			continue
		}

		switch i % 4 {
		case 0:
			out[j] = ch << 2
		case 1:
			out[j] |= ch >> 4
			j++
			out[j] = (ch & 0x0f) << 4
		case 2:
			out[j] |= ch >> 2
			j++
			out[j] = (ch & 0x03) << 6
		case 3:
			out[j] |= ch
			j++
		}
		i++
	}

	k := make([]byte, 32)
	ky := []byte(key)
	for i, v := range ky {
		k[i] = v
	}
	cb, err := aes.NewCipher(k)
	if err != nil {
		return nil, fmt.Errorf(`aes.NewCipher: %v`, err)
	}

	if len(out) < 16 {
		return nil, errors.New("数据结构不对")
	}
	var resp = make([]byte, len(out[16:]))
	if len(resp) >= len(out[16:j]) {
		cipher.NewCFBDecrypter(cb, out[0:16]).XORKeyStream(resp, out[16:j])
	} else {
		return nil, errors.New("数据结构不对")
	}

	return resp, nil
}
